#!/usr/bin/python
# -*- coding: utf-8 -*-

__title__ = 'SauceNAO'
__description__ = 'Small module to work with SauceNAO locally'
__version__ = '1.0.1'
__url__ = 'https://github.com/DaRealFreak/saucenao'
__author__ = 'DaRealFreak'
__author_email__ = 'steffen.keuper@web.de'
__license__ = 'MIT'
